#!/bin/bash
####################################################################
#
# This script is invoked by the Bulk Data Transfer CLI to perform
# various operations on  block devices.  It is not intended to be
# run manually. 
#
# >>>>>>>>>>>>>>> DO NOT EXECUTE THIS SCRIPT <<<<<<<<<<<<<<<<
#
###################################################################
DEVICE_LABEL_PREFIX_HDD="orcdts_"
DEVICE_LABEL_PREFIX_ISCSI="dts"
MOUNT_PREFIX="/mnt/"
MAPPER_PREFIX="/dev/mapper/"

usage() {
  # empty
  echo "DO NOT EXECUTE THIS SCRIPT"
}

validate_arg() {
  echo $1 | grep "^\-" >&2
  if [ $? -eq 0 ];then
   usage
    exit -1
  fi
}

# Functions set this to indicate return value
STATUS=0

# Determine if the label set in $label matches any mountpoint
# in the list of mount points returned from the 'mount' command.
# For reference:
# /dev/mapper/beaf11 on /mnt/dtsDQ2MX5QPF type xfs (rw,relatime,attr2,inode64,noquota)
# field 3 has the mount point. So matching the label which starts with 'D' if found
# yields the mount path.
mountpoint() {
  if [ -z "$label" ];then
    usage
    exit -1
  fi
  MOUNTPOINT=$(mount | awk '{ print $3 }' |grep $label)
  if [ $? -eq 0 ];then
    echo $MOUNTPOINT
  else
    >2& echo "device $label is not mounted"
    exit -1
  fi
}


query_devices() {
  if [ -n "$device" ];then
    if [ ! -b $device ];then
      >&2 echo "$device is not a valid block device"
      exit -1
    fi
    lsblk -o name,size,type,uuid,label,mountpoint $device
  else
    lsblk -o name,size,type,uuid,label,mountpoint
  fi
}

# sets NUM_PARTITIONS
get_num_partitions() {
  local device=$1
  dev_name=`basename $device`
  NUM_PARTITIONS=`lsblk -l -o name | grep -c "${dev_name}[0-9][0-9]*"`
}

# sets STATUS
# Check for a filesystem on the device.  Assumes no partitions.
# STATUS:
#   0 : No filesystem
#   1 : Filesystem, not encrypted
#   2 : Filesystem, encrypted but not created by DTS
#   3 : Filesystem, encrypted and created by DTS
check_filesystem() {
  local device=$1
  local short_device_name=`basename $device`
  uuid=`lsblk -l -o name,uuid | grep $short_device_name | awk '{ print $2 }'`
  if [ -n "$uuid" ];then
    # there is a filesystem
    fs_type=`lsblk -l -o name,type | grep $short_device_name | awk '{ print $2 }'`
    if [ $fs_type = "crypt" ];then
      # filesystem is encrypted
      fs_label=`lsblk -l -o name,label | grep $short_device_name | awk '{ print $2 }'`
      # does the file system label start with our prefix, "orcldts_" for ext4 or "dts" for xfs?
      echo $fs_label | grep -q-e "^$DEVICE_LABEL_PREFIX_HDD" -e "^$DEVICE_LABEL_PREFIX_ISCSI"
      if [ $? -eq 0 ];then
        STATUS=3
      else
        STATUS=2
      fi
    else
      STATUS=1
    fi
  else
    STATUS=0
  fi
}

verify_device() {
  if [ -z "$device" ];then
    usage
    exit -1
  fi
  if [ ! -b $device ];then
    >&2 echo "$device is not a valid block device."
    exit -1
  fi
  get_num_partitions $device
  if [ $NUM_PARTITIONS -ge 1 ];then
    >&2 echo "There are $NUM_PARTITIONS partitions on $device.  You must remove all partitions."
    exit -1
  fi
  check_filesystem $device
  case $STATUS in
  0)
    exit 0
    ;;
  1)
    >&2 echo "There is a filesystem on $device.  Unable to proceed."
    exit -1
    ;;
  2)
    >&2 echo "There is an encrypted filesystem on $device.  Unable to proceed."
    exit -1
    ;;
  3)
    >&2 echo "$device has already been set up by this script with an encrypted filesystem."
    exit -1
    ;;
  esac
}
 
mount_filesystem() {

#We need these parameters here:
# device - the block deivce like /dev/sdb, /dev/sdc
# mount_label - the dir name under /mnt for this device.
# fstype - either ext4 or xfs. Depends on whether it's locally attached or iSCSI
# serial_number - read with another utility function and stored as an attribute of the device.
  if [ -z $device ]; then
    echo "block device not provided."
    usage
    exit 0
  fi
  if [ -z $mount_label ]; then
    echo "mount label not provided."
    usage
    exit -1
  fi
  if [ -z $fstype ]; then
    echo "file system type not provided."
    usage
  fi
  if [ -z $serial_number ];then
    echo "serial number not provided."
    usage
  fi

  local mount_point

  MAPPED_DEVICE="${MAPPER_PREFIX}${serial_number}"
  MOUNTPOINT="/mnt/${mount_label}"
  mkdir -p $MOUNTPOINT
  if [ $? -ne 0 ];then
    >&2 echo "failed to create mount directory $MOUNTPOINT"
    exit -1
  fi
  umount -f $MOUNTPOINT 2> /dev/null

# The mount should be for ext4 or XFS. If it's XFS we need to mount with the option of
# -o
  if [ "$fstype" == "ext4" ]; then
    mount $MAPPED_DEVICE $MOUNTPOINT
  elif [ "$fstype" == "xfs" ]; then
    mount -o inode64 $MAPPED_DEVICE $MOUNTPOINT
  else
    echo "Only ext4 and xfs mounts are supported"
    exit -1
  fi

  if [ $? -ne 0 ];then
    >&2 echo "failed to mount encrypted device $device at $MOUNTPOINT"
    exit -1
  fi
}

lock_device() {
  if [ -z "$device" ]; then
    echo "block device not provided."
    exit -1
  fi
  if [ -z "$label" ]; then
    echo "device label not provided."
    exit -1
  fi

  if [ ! -b $device ];then
    >&2 echo "$device is not a valid block device"
    exit -1
  fi
  # Examine the output of a mount and look for some mount point that includes the label for this device.
  # The full mount point is of the form /mnt/orcl_dtsDXXXXXX or /mnt/dts_DXXXXXXX so looking for the
  # device label is sufficient.
  mount | grep -q "$label"
  # If we get a success then then device mapper name can be extracted from the mount listing.
  # The listing will be of the form
  # /mnt/orcldts_DXXXXXX /dev/mapper/serial-number
  # so the second string will be the device mapper name we need for crpytsetup

  if [ $? -eq 0 ];then
    mount_point=`mount | grep "$label" | awk '{ print $3 }'`
    dm_name=`mount | grep "$label" | awk '{ print $1 }'`
    umount -f $mount_point
  else
    >&2 echo "device $label $device is not mounted"
    >&2 echo "it may be locked already"
    exit -1
  fi
  cryptsetup luksClose $dm_name
  if [ $? -ne 0 ];then
    >&2 echo "failed to lock encrypted device $device $label"
    exit -1
  fi
  echo "$device $label is encrypted and locked"
}


########## Main ############

device=""
label=""
mount_label=""
fstype=""
serial_number=""
progname=`basename $0`

# Any letter followed by a ':' means that option is followed by a value. So a: would be
# -a foo.
while getopts "qd:s:p:P:l:t:m:h" opt; do
  validate_arg $OPTARG
  case $opt in
    d)
      device=$OPTARG
      ;;
    l)
      label=$OPTARG
      ;;
    s)
      serial_number=$OPTARG
      ;;
    t)
      fstype=$OPTARG
      fstype=`echo "$fstype" | awk '{ print tolower($1) }'`
      ;;
    m)
      mount_label=$OPTARG
      ;;
    h)
      usage
      exit 0
      ;;
    \?)
      echo "Invalid option: -$OPTARG" >&2
      exit 1
      ;;
    :)
      echo "Option -$OPTARG requires an argument." >&2
      exit 1
      ;;
  esac
done

shift $((OPTIND-1))

operation=$1

# Each operation will validate that it has the necessary options set.
case "$1" in
  query)
    query_devices
    ;;
  verify)
    verify_device
    ;;
  mount)
    mount_filesystem
    ;;
  mountpoint)
    mountpoint
    ;;
  lock)
    lock_device
    ;;
  *)
    usage
    exit -1
 
esac

